// Definitions for panel-one only

@import url("../panels.less"); // this import statement will be kept in the output

#$module-panel-one {
  background: #fffee0;
}

#$module-panel-one h2 {
  color: @text-color;
  margin: 0;
}
